"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const child_process_1 = require("child_process");
const fs_1 = require("fs");
const path_1 = require("path");
const pExec = (command) => new Promise((resolve, reject) => {
    (0, child_process_1.exec)(command, (error, stdout, stderr) => {
        if (stderr) {
            console.error(`stderr: ${stderr}`);
        }
        if (error) {
            reject(error);
        }
        resolve(stdout);
    });
});
const getAllFiles = function (dirPath, arrayOfFilesInput = []) {
    const files = (0, fs_1.readdirSync)(dirPath);
    let arrayOfFiles = arrayOfFilesInput;
    files.forEach(function (file) {
        if ((0, fs_1.statSync)(dirPath + '/' + file).isDirectory()) {
            arrayOfFiles = getAllFiles(dirPath + '/' + file, arrayOfFiles);
        }
        else {
            arrayOfFiles.push((0, path_1.join)(dirPath, '/', file));
        }
    });
    return arrayOfFiles;
};
const TMP_FOLDER = '/tmp/serverless-esbuild-bundle-analyzer';
function bundleVisualizer() {
    return __awaiter(this, void 0, void 0, function* () {
        const { analyze: functionName } = this.options;
        if (functionName === undefined) {
            return;
        }
        const functionZipName = this.serverless.service
            .getAllFunctionsNames()
            .find(fullFunctionName => fullFunctionName.endsWith(functionName));
        if (functionZipName === undefined) {
            this.serverless.cli.log(`🤯 Analyze failed: function ${functionName} was not found`, 'ServerlessAnalyzeBundlePlugin', { color: 'red' });
            return;
        }
        this.serverless.cli.log(`⏳ Analyzing function ${functionName}`, 'ServerlessAnalyzeBundlePlugin');
        const TEMP_DIR_LOCATION = `${TMP_FOLDER}/${functionZipName}/${new Date().getTime()}`;
        yield pExec(`mkdir -p ${TEMP_DIR_LOCATION}`);
        yield pExec(`unzip .serverless/${functionZipName} -d ${TEMP_DIR_LOCATION}`);
        const test = getAllFiles(`${TEMP_DIR_LOCATION}`);
        const metafileName = test.filter(fileName => fileName.includes(`/${functionName}/`) && fileName.endsWith('-meta.json'))[0];
        yield pExec([
            'node_modules/.bin/esbuild-visualizer',
            '--metadata',
            metafileName,
            '--filename',
            `${TEMP_DIR_LOCATION}/${functionName}.html`,
            '--open',
        ].join(' '));
    });
}
exports.default = bundleVisualizer;
